@@ -179,7 +179,7 @@ namespace SDDM {
             QDir().mkpath(finfo.absolutePath());
 
             QFile file_handler(file);
-            file_handler.open(QIODevice::WriteOnly);
+            file_handler.open(QIODevice::Append);
             file_handler.close();
 
             QString cmd = QStringLiteral("%1 -f %2 -q").arg(mainConfig.X11.XauthPath.get()).arg(file);
@@ -198,4 +198,13 @@ namespace SDDM {
             pclose(fp);
         }
     }
+
+    void UserSession::setCachedProcessId(qint64 pid) {
+        m_cachedProcessId = pid;
+    }
+
+    qint64 UserSession::cachedProcessId() {
+        return m_cachedProcessId;
+    }
+
 }
